#!/usr/bin/env python3


'''
Application example of the Fnetout class, based on an output
file that provides the network predictions and targets, relevant
information are extracted and printed to the standard output.
'''


from fortformat import Fnetout


def main():
    '''Main driver routine.'''

    fnetout = Fnetout('fnetout.hdf5')

    mode = fnetout.mode
    print('Running mode:\n', mode)

    ndatapoints = fnetout.ndatapoints
    print('Number of datapoints in dataset:\n', ndatapoints)

    nglobaltargets = fnetout.nglobaltargets
    print('Number of system-wide targets (e.g. total energies):\n',
          nglobaltargets)

    natomictargets = fnetout.natomictargets
    print('Number of atomic targets (e.g. atomic forces):\n',
          natomictargets)

    globaltargets = fnetout.globaltargets
    print('System-wide targets:\n', globaltargets)

    atomictargets = fnetout.atomictargets
    print('Atomic targets: ', atomictargets)

    tforces = fnetout.tforces
    print('Whether atomic forces are present:\n', tforces)

    forces = fnetout.forces
    print('Atomic forces:\n', forces)

    atomicpredictions = fnetout.atomicpredictions
    print("Fortnet's predictions of atomic targets:\n", atomicpredictions)

    globalpredictions = fnetout.globalpredictions
    print("Fortnet's predictions of system-wide targets:\n", globalpredictions)

    globalpredictions_atomic = fnetout.globalpredictions_atomic
    print("Fortnet's atom-resolved predictions of system-wide targets:\n",
          globalpredictions_atomic)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of the Fortformat package, based on an output
file that contains network predictions and corresponding targets.
'''


import numpy as np
import matplotlib.pyplot as plt
from fortformat import Fnetout


def main():
    '''Main driver routine.'''

    nndists = np.arange(2.10, 3.30 + 0.05, 0.05)

    fnetout = Fnetout('fnetout.hdf5')
    globalpredictions = fnetout.globalpredictions
    globaltargets = fnetout.globaltargets

    plt.figure(figsize=(7, 5))
    plt.title('Comparison of Neural Network Predictions with Targets')
    plt.xlabel(r'Nearest Neighbour Distance [$\mathrm{\AA}$]')
    plt.ylabel('Total Energy [eV / Atom]')

    plt.plot(nndists, globalpredictions / 2.0, color='blue', label='NN')
    plt.scatter(nndists, globaltargets / 2.0, s=10, color='black', label='DFT')

    plt.tight_layout()
    plt.legend()
    plt.savefig('comparison.svg', dpi=900, format='svg')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of an H2 geometry optimization by invoking
Fortnet via the file-IO interface to the ASE Python package.
'''


from ase.build import molecule
from ase.optimize import BFGS
from fnetase import Fortnet


def main():
    '''Main driver routine.'''

    system = molecule('H2')

    # perturb H2 molecule
    system.positions[0] = system.positions[0] * 1.3
    system.positions[1] = system.positions[1] * 1.3

    calc = Fortnet(label='H2', atoms=system, restart='fortnet.hdf5')

    system.calc = calc

    opt = BFGS(system, trajectory='opt.traj', logfile='opt.log')
    opt.run(fmax=1.0e-006)

    energy = system.get_potential_energy()
    print(energy)
    forces = system.get_forces()
    print(forces)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of the Fnetdata class, based on a dataset
that provides global system properties as target values to fit.
The entire dataset is written to a contiguous fnetdata.hdf5 file,
whereby atomic gradient contributions are weighted by user-input.
'''


import os
import numpy as np
from fortformat import Fnetdata
from ase.io.vasp import read_vasp, read_vasp_out


def main():
    '''Main driver routine.'''

    inpaths = [os.path.join(os.getcwd(), '../../globalTargets/vaspdata', entry)
               for entry in sorted(os.listdir('../../globalTargets/vaspdata'))]

    # fix random seed for reproduction purposes
    np.random.seed(42)

    strucs = []
    atomicweights = []
    energies = np.empty((len(inpaths), 1))

    for ii, inpath in enumerate(inpaths):
        struc = read_vasp(os.path.join(inpath, 'POSCAR'))
        strucs.append(struc)
        props = read_vasp_out(os.path.join(inpath, 'OUTCAR'))
        energies[ii, 0] = props.get_total_energy()
        # float-valued atomic gradient weighting in interval [1, 10]
        atomicweights.append(np.asfarray(
            np.random.randint(1, 10, len(struc), dtype=int)))

    fnetdata = Fnetdata(atoms=strucs, globaltargets=energies)
    fnetdata.atomicweights = atomicweights
    fnetdata.dump('fnetdata_float_weighting.hdf5')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of a single-point calculation by invoking
Fortnet via the file-IO interface to the ASE Python package.
'''


from ase.build import molecule
from fnetase import Fortnet


def main():
    '''Main driver routine.'''

    system = molecule('H2')

    calc = Fortnet(label='H2', atoms=system, restart='fortnet.hdf5')

    calc.calculate(atoms=system, properties=('energy', 'forces'))

    energy = system.get_potential_energy()
    forces = system.get_forces()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of the Fnetdata class, based on a dataset
that provides global system properties as target values to fit.
The dataset is extended by user specified external atomic features.
'''


import os
import numpy as np
from fortformat import Fnetdata
from ase.io.vasp import read_vasp, read_vasp_out


def main():
    '''Main driver routine.'''

    np.random.seed(42)

    inpaths = [os.path.join(os.getcwd(), '../globalTargets/vaspdata', entry)
               for entry in sorted(os.listdir('../globalTargets/vaspdata'))]

    strucs = []
    features = []
    energies = np.empty((len(inpaths), 1))

    for ii, inpath in enumerate(inpaths):
        struc = read_vasp(os.path.join(inpath, 'POSCAR'))
        strucs.append(struc)
        props = read_vasp_out(os.path.join(inpath, 'OUTCAR'))
        energies[ii, 0] = props.get_total_energy()
        features.append(np.random.random_sample((len(struc), 3)))

    fnetdata = Fnetdata(atoms=strucs, globaltargets=energies,
                        features=features)
    fnetdata.dump('fnetdata.hdf5')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

'''
Application of the Fortformat package, based on an output file that contains
atomic forces, resulting from analytical expressions or finite differences.
'''


import numpy as np
from fortformat import Fnetout


def main():
    '''Main driver routine.'''

    fnetout = Fnetout('fnetout.hdf5')
    forces = fnetout.forces

    # print forces of each datapoint, network
    # output and atom to illustrate the indexing:
    for idata in range(len(forces)):
        for iout in range(len(forces[idata])):
            for iatom in range(np.shape(forces[idata][iout])[0]):
                print(forces[idata][iout][iatom])


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of the Fnetdata class, based on a dataset
that provides global system properties as target values to fit.
'''


import os
import numpy as np
from fortformat import Fnetdata
from ase.io.vasp import read_vasp, read_vasp_out


def main():
    '''Main driver routine.'''

    nndists = np.arange(2.00, 3.50 + 0.05, 0.05)

    inpaths = [os.path.join(os.getcwd(), 'vaspdata', entry)
               for entry in sorted(os.listdir('vaspdata'))]

    strucs = []
    energies = np.empty((len(inpaths), 1))

    for ii, inpath in enumerate(inpaths):
        strucs.append(read_vasp(os.path.join(inpath, 'POSCAR')))
        props = read_vasp_out(os.path.join(inpath, 'OUTCAR'))
        energies[ii, 0] = props.get_total_energy()

    fnetdata = Fnetdata(atoms=strucs, globaltargets=energies)
    fnetdata.dump('fnetdata.hdf5')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


'''
Application example of an H2O geometry optimization by invoking
Fortnet via the socket interface to the ASE Python package.
'''

import sys
from subprocess import Popen
from ase.io import read, write
from ase.optimize import BFGS
from ase.io.trajectory import Trajectory
from ase.calculators.socketio import SocketIOCalculator


UNIXSOCKET = 'fortnet'
FNET = 'fnet'
GEO_IN = './H2O_unoptimized.xyz'
GEO_OUT = './H2O_optimized.xyz'
TRAJECTORY = 'opt.traj'


def main():
    '''Main driver routine.'''

    system = read(GEO_IN, format='xyz')

    opt = BFGS(system, trajectory=TRAJECTORY, logfile='opt.log')

    with SocketIOCalculator(log=sys.stdout, unixsocket=UNIXSOCKET) as calc:
        Popen(FNET)
        system.set_calculator(calc)
        opt.run(fmax=1E-04)

    traj = Trajectory(TRAJECTORY)
    atoms = traj[-1]
    write(GEO_OUT, atoms, format='xyz')

if __name__ == "__main__":
    main()



#!/usr/bin/env python3


'''
Application example of an H2 molecular dynamics by invoking
Fortnet via the file-IO interface to the ASE Python package.
'''


from ase.build import molecule
from ase.io.trajectory import Trajectory
from ase.md.langevin import Langevin
from ase.md.velocitydistribution import (MaxwellBoltzmannDistribution,
                                         Stationary, ZeroRotation)
from ase import units
from fnetase import Fortnet


def main():
    '''Main driver routine.'''

    system = molecule('H2')

    system.calc = Fortnet(label='H2', atoms=system, restart='fortnet.hdf5')

    MaxwellBoltzmannDistribution(system, temperature_K=200)
    Stationary(system)
    ZeroRotation(system)

    dyn = Langevin(system, 1.0 * units.fs, friction=1e-02, temperature_K=200)

    def printenergy(atoms=system):
        '''Prints the potential, kinetic and total energy.'''
        epot = atoms.get_potential_energy() / len(atoms)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        print('Energy per atom: Epot = %.3feV  Ekin = %.3feV (T=%3.0fK)  '
              'Etot = %.3feV' % (epot, ekin, ekin / (1.5 * units.kB),
                                 epot + ekin))

    dyn.attach(printenergy, interval=10)

    traj = Trajectory('md.traj', 'w', system)
    dyn.attach(traj.write, interval=1)

    printenergy()
    dyn.run(200)


if __name__ == '__main__':
    main()
